#!/usr/bin/python2

from __future__ import print_function

import json
import os
import os.path
import subprocess
import sys
import urllib


job_template = '%s/job/%s/lastSuccessfulBuild/api/json'
artifacts_template = '%s/job/%s/lastSuccessfulBuild/artifact/%s'

store_uri = 'https://search.apps.ubuntu.com/api/v1/package/%s'

click_packages = [
    'calculator-app-click',
    'calendar-app-click',
    'clock-app-click',
    'dropping-letters-click',
    'notes-app-click',
    'rssreader-app-click',
    'stock-ticker-mobile-app-click',
    'sudoku-app-click',
    'weather-app-click',
    ]

click_packages_staged = [
    'filemanager-app-click',
    'music-app-click',
    'rssreader-app-click',
    'terminal-app-click',
    'camera-app-click',
    ]

store_click_packages = [
    'com.ubuntu.developer.webapps.webapp-amazon',
    'com.ubuntu.developer.webapps.webapp-ebay',
    'com.ubuntu.developer.webapps.webapp-facebook',
    'com.ubuntu.developer.webapps.webapp-gmail',
    'com.ubuntu.developer.webapps.webapp-twitter',
    'com.ubuntu.developer.webapps.webapp-ubuntuone',
    ]


def get_json_for_uri(click_uri):
    """
    Returns a json object for the latest successful job from the
    Jenkins uri.
    """
    print(('Getting %s' % click_uri))
    json_job_request = urllib.urlopen(click_uri).read()
    json_content = json.loads(json_job_request)
    return json_content


def remove_if_previous(click_package, target_dir):
    """
    Locates click_package in target directory and removes it.
    """
    dir_contents = os.listdir(target_dir)
    click_package.split('_')[0]
    matches = filter(lambda x: click_package in x, dir_contents)
    for match in matches:
        target = os.path.join(target_dir, match)
        if os.path.exists(target):
            print('Removing previous click package %s' % match)
            os.unlink(target)


def download(uri, target_file):
    """
    Downloads uri to target_file.
    """
    print(('Downloading %s to %s' % (uri, target_file)))
    new_target_file = "%s.new" % target_file
    try:
        subprocess.check_call(['wget', uri, '-O', new_target_file])
    except subprocess.CalledProcessError:
        if os.path.exists(new_target_file):
            os.unlink(new_target_file)
        raise
    os.rename(new_target_file, target_file)


def get_store_apps(download_dir):
    """
    Gets click apps from the store.
    """
    download_list = []
    for click in store_click_packages:
        json_content = get_json_for_uri(store_uri % click)
        click_file = json_content['download_url'].split('/')[-1]
        target_file = os.path.join(download_dir, click_file)
        download(json_content['download_url'] + '?noauth=1', target_file)
        download_list.append(click_file)
    return download_list


if __name__ == '__main__':
    if len(sys.argv) != 3:
        print('Jenkins uri base and target dir argument expected')
        exit(1)
    jenkins_base_uri = sys.argv[1]
    download_dir = sys.argv[2]
    jenkins_uri = jenkins_base_uri
    download_list = []
    for click_package in click_packages:
        url = job_template % (jenkins_uri, click_package)
        json_job = get_json_for_uri(url)
        click_package_uri = (artifacts_template %
                             (jenkins_base_uri,
                              click_package,
                              json_job['artifacts'][0]['relativePath']))
        click_files = [x['fileName'] for x in json_job['artifacts']
                       if x['fileName'].endswith('.click')]
        for click_file in click_files:
            target_file = os.path.join(download_dir, click_file)
            remove_if_previous(click_file, download_dir)
            download(click_package_uri, target_file)
            download_list.append(click_file)
    download_list = download_list + get_store_apps(download_dir)
    click_list = os.path.join(download_dir, 'click_list')
    with open(click_list, 'w') as f:
        for i in download_list:
            f.write('%s\n' % i)
